function [ transformed_data inv_transformation_matrix ] =...
    coordinate_transform(...
    data,primary_coordinate,secondary_coordinate,definitions)
%% The purpose of this function is to transform a data set's axes.
% Primary coordinate will define an axis
    %(which axis is defined by "definitions")
% Secondary coordinate will define a plane
    %(which plane is defined by "definitions")
% Definitions is a cell with 2 strings that defines the axes
    %(options are 'x','y',or 'z') 

%%  Input errors
%data formatting
if size(data,2)                ~=3; error('data must have 3 columns' ); end
if size(primary_coordinate,  2)~=3; error('coordinates must be n x 3'); end
if size(secondary_coordinate,2)~=3; error('coordinates must be n x 3'); end
if  size(primary_coordinate, 1)~=size(data,1) && ...
        size(secondary_coordinate,1)~=size(data,1);
    error('length of data must equal length of coordinates');
end
% "definitions" error inputs
if sum(size(definitions))~=3; ...
        error('definitions must contain 2 strings');end
if iscellstr(definitions)~=1; ...
        error('definitions must be a cell with strings'); end

%% Interpret "definitions"
%primary component
if     strcmp(definitions{1},'x')==1; p=1;
elseif strcmp(definitions{1},'y')==1; p=2;
elseif strcmp(definitions{1},'z')==1; p=3;
end
%secondary component
if     strcmp(definitions{2},'x')==1; s=1;
elseif strcmp(definitions{2},'y')==1; s=2;
elseif strcmp(definitions{2},'z')==1; s=3;
end
%tertiary component
t=6-p-s;

%% Determine sign of cross-product of primary and secondary vectors
%That is cross(x,y)=z, but cross(y,x)=-z
if p-s==-1 || p-s==2;
    sign=1;
elseif p-s==1 || p-s==-2;
    sign=-1;
end

%% If primary or secondary vector are constant
    %duplicate their rows
%primary vector duplication    
if size(primary_coordinate,1)==1;
    primary_coordinate=repmat(primary_coordinate,...
        size(data,1),1);
end
%secondary vector duplication
if size(secondary_coordinate,1)==1;
    secondary_coordinate=repmat(secondary_coordinate,...
        size(data,1),1);
end

%% Create unit vectors
%create variables in memory
primary_unit  =NaN(size(data,1),3);
secondary_unit=NaN(size(data,1),3);
tertiary_unit =NaN(size(data,1),3);
for i=1:size(primary_coordinate,1)
    %convert new axes vectors into unit vectors (definition: magnitude 1)
    %The unit vector that describes the primary axis
    primary_unit  (i,:)=primary_coordinate(i,:)/...
        sqrt(dot(primary_coordinate(i,:),  primary_coordinate(i,:)));
    %This unit vector is not necessarily perpendicular to the primary axis,
        %but it is within the plane defined by the primary and secondary v.
    secondary_temp_unit=secondary_coordinate(i,:)/...
        sqrt(dot(secondary_coordinate(i,:),secondary_coordinate(i,:)));
    %The third axis is perpendicular to the plane defined by the
        %primary and secondary_temp unit vectors
        % "sign" restricts it so the 'right hand rule' is obeyed.
    tertiary_unit (i,:)=   sign.*cross(...
        primary_unit(i,:),secondary_temp_unit);
    %create the secondary unit vector
    secondary_unit(i,:)=-1*sign.*cross(...
        primary_unit(i,:),tertiary_unit(i,:));
    %re-scale secondary and teriary unit, due to computational imprecision
    secondary_unit (i,:)=secondary_unit (i,:)/...
        sqrt(dot(secondary_unit(i,:),secondary_unit(i,:)));
    tertiary_unit (i,:)=tertiary_unit (i,:)/...
        sqrt(dot(tertiary_unit(i,:),tertiary_unit(i,:)));
end

%% Transform matrix
%create variables in memory
transformation_matrix=NaN(3,3);
inv_transformation_matrix=NaN(3,3,size(data,1));
transformed_data=NaN(size(data));
for i=1:size(data,1)
    for k=1:size(data,3)
        %create transformation matrix
        transformation_matrix(:,p)=primary_unit  (i,:)';
        transformation_matrix(:,s)=secondary_unit(i,:)';
        transformation_matrix(:,t)=tertiary_unit (i,:)';
        %transform data
        transformed_data(i,:,k)=...
            data(i,:,k)*transformation_matrix;
        %Inverse transformation matrix, for vector*matrix=new_vector
        inv_transformation_matrix(:,:,i)=...
            transformation_matrix^-1;
    end
end

