%% Kinematic Analysis for RedBlue Gust017

% Part 1: Load digitized data
% Part 2: Transform data into body coordinate system
% Part 3: Compute kinematics
% Part 4: Generate kinematics plot

% 3D Data Legend:
% Point #   Landmark   Columns
%    1        shdL       1:3
%    2        shdR       4:6
%    3        wstL       7:9
%    4        wstR      10:12
%    5        elbL      13:15
%    6        elbR      16:18
%    7         t3L      19:21
%    8         t3R      22:24
%    9         t4L      25:27
%    10        t4R      28:30
%    11        t5L      31:33
%    12        t5R      34:36
%    13       mcp3L     37:39
%    14       mcp3R     40:42
%    15       mcp4L     43:45
%    16       mcp4R     46:48
%    17       mcp5L     49:51
%    18       mcp5R     52:54
%    19       ankL      55:57
%    20       ankR      58:60
%    21       lmb       61:63
%    22       str       64:66

%% Load Raw Position Data
clear
close all
trl = 'RedBlue Gust017';
% 
% [d time] = Load_Old_Points('/Volumes/Bat_Durip/Gust Response/Analysis/RedBlue/Gust017/');
% csvwrite('RB_g017_xyzpts.csv',d)
d_unfilt=csvread('/Users/davidboerma/Google Drive/Gust Response/Digitizing/RedBlue/RedBlue_fromPythonTracker/RB_g017_xyzpts_ALL.csv',1,0);

gust = [40 69]; 

% Filter Position Data 
d = NaN(size(d_unfilt));
f1 = 32;
f2 = 182;
cutoff_freq = 100;
sample_freq = 800;
[p1,p2] = butter(2,cutoff_freq/(sample_freq/2),'low');
d(f1:f2,:) = filtfilt(p1,p2,d_unfilt(f1:f2,:));

d(1,67:78) = d_unfilt(1,67:78);

% Trim frames to eliminate NaNs (Pre-trimmed for this trial)
% d = data(:,:);

%%
        % Plot motion traces to check data
        figure
        f=1:240;
        for i=1:22
            scatter3(d(f,i*3-2),d(f,i*3-1),d(f,i*3),20,'filled')
            hold on
        end
        hold on
        plot3([d(f,1) d(f,4)],[d(f,2) d(f,5)],[d(f,3) d(f,6)],'k')
        plot3([d(f,4) d(f,61)],[d(f,5),d(f,62)],[d(f,6),d(f,63)],'k')
        plot3([d(f,61),d(f,1)],[d(f,62) d(f,2)],[d(f,63) d(f,3)],'k')
        axis equal

%% Express Data in Local Coordinate System
[ LCSdata ] = global2Local( d );


%% Kinematic Analysis
[ wngext_L, wngext_R, elev_L, elev_R, prot_L, prot_R, elbL, elbR, dang3_radL, dang3_radR ] = KinematicsCalculations( LCSdata );

% Normalize wing extension to max extension for each wing (mean max extension over all trials for the individual).
load('/Users/davidboerma/Google Drive/Gust Response/Analysis/RB_MaxWngExt.mat');
extNorm_L = wngext_L / RB_MaxWngExt(1);
extNorm_R = wngext_R / RB_MaxWngExt(2);

[ AoAL, AoAR, phiL, phiR ] = AoA(d);

%% Calculate Changes in Body Orientation
[ r, p, y, rv, pv, yv ] = Calc_Body_Orientation_IndivGlobal(d);

%
figure, hold on, plot(r, 'r')
plot(y, 'g')
plot(p, 'b')

%% Normalize Kinematic data to the wingbeat cycle
% Stroke Definitions: Define Up- and Downstrokes
SD = [35 92 115 143 170];

% Build master kinematics matrix
Kinematics(:,1) = extNorm_L;
Kinematics(:,2) = extNorm_R;
Kinematics(:,3) = elev_L;
Kinematics(:,4) = elev_R;
Kinematics(:,5) = prot_L;
Kinematics(:,6) = -prot_R;
Kinematics(:,7) = elbL;
Kinematics(:,8) = elbR;
Kinematics(:,9) = dang3_radL;
Kinematics(:,10) = dang3_radR;
Kinematics(:,11) = r;
Kinematics(:,12) = p;
Kinematics(:,13) = y;
Kinematics(:,14) = rv;
Kinematics(:,15) = pv;
Kinematics(:,16) = yv;
Kinematics(gust(1):gust(2),17) = 10;
Kinematics(:,18) = rad2deg(AoAL);
Kinematics(:,19) = rad2deg(AoAR);

% Normalize Kinematic data to wingstrokes
for i=1:size(Kinematics,2)
[norm_Kinematics(:,i), norm_domain(:,i)] = norm_data(Kinematics(:,i), SD, 1, 'c');
end

nk = norm_Kinematics;
nd = norm_domain(:,1);

%% plot some wingbeat-normalize kinematics
n=7;
figure
% Roll
subplot(n,1,1)
plot(nd, nk(:,11),'r','linewidth',2), hold on
plot(nd(:,1),nk(:,17),'k')
title({trl;'Roll'})

% Humeral Elevation
subplot(n,1,2)
plot(nd,nk(:,3),'b','linewidth',2), hold on
plot(nd,nk(:,4),'r','linewidth',2)
title('Humeral Elevation')
legend('Left','Right')

% Humeral Protraction
subplot(n,1,3)
plot(nd, nk(:,5),'b','linewidth',2), hold on
plot(nd, nk(:,6),'r','linewidth',2)
title('Humeral Protraction')

% Wing Extension
subplot(n,1,4)
plot(nd, nk(:,1),'b','linewidth',2), hold on
plot(nd, nk(:,2),'r','linewidth',2)
title('Wing Extension')
legend('Left','Right')

% Elbow Extension
subplot(n,1,5)
plot(nd, nk(:,7),'b','linewidth',2), hold on
plot(nd, nk(:,8),'r','linewidth',2)
title('Elbow Extension')
legend('Left','Right')

% Handwing Abduction
subplot(n,1,6)
plot(nd, nk(:,9),'b','linewidth',2), hold on
plot(nd, nk(:,10),'r','linewidth',2)
title('Digit Abduction')
legend('Left','Right')

% Angle of Attack
subplot(n,1,7)
plot(nd, nk(:,18), 'b','linewidth',2), hold on
plot(nd, nk(:,19), 'r','linewidth',2)
title('Angle of Attack')
legend('Left', 'Right')

%% plot some non-normalized kinematics
figure
kn=Kinematics;
n=7;
x=[f1-10,f2+10];
% Rotations
subplot(n,1,1)
plot(kn(:,11),'r','linewidth',2),hold on
plot(kn(:,12),'b','linewidth',2)
plot(kn(:,13),'g','linewidth',2)
title('Rotations')
xlim(x)

% Humeral Elevation
subplot(n,1,2)
plot(kn(:,3),'b','linewidth',2),hold on
plot(kn(:,4),'r','linewidth',2)
title('Humeral Elevation')
xlim(x)

% Humeral Protraction
subplot(n,1,3)
plot(kn(:,5),'b','linewidth',2),hold on
plot(kn(:,6),'r','linewidth',2)
title('Humeral Protraction')
xlim(x)

% Wing Extension
subplot(n,1,4)
plot(kn(:,1),'b','linewidth',2),hold on
plot(kn(:,2),'r','linewidth',2)
title('Wing Extension')
xlim(x)

% Elbow Angle
subplot(n,1,5)
plot(kn(:,7),'b','linewidth',2),hold on
plot(kn(:,8),'r','linewidth',2)
title('Elbow angle')
xlim(x)

% Handwing Angle
subplot(n,1,6)
plot(kn(:,9),'b','linewidth',2),hold on
plot(kn(:,10),'r','linewidth',2)
title('Handwing Angle')
xlim(x)

% Angle of Attack
subplot(n,1,7)
plot(kn(:,18),'b','linewidth',2), hold on
plot(kn(:,19),'r','linewidth',2)
title('Angle of Attack')
xlim(x)



%% Calculate Symmetry (Standard Deviation of Difference Between Left/Right Wings)
% Define Stroke Periods
wb = [35 92; 92 115; 115 143; 143 170];

wb_avg = round(mean(wb(:,2)-wb(:,1)));
w = wb_avg;

% Find asymmetry between left and right wings for each kinematic
% parameter using sqrt(mean(l/r difference)^2)) . . . this calculation is
% from Kenny. Std may not be best since it subtracts the mean of my data
% over the period defined by my window instead of subtracting a more representative baseline.
    % asym(:,1) = wing extension
    % asym(:,2) = humeral elevation, i.e. elevation
    % asym(:,3) = humeral protraction/retracion, i.e. sweep
    % asym(:,4) = elbow angle
    % asym(:,6) = angle of attack

numvar = 6; % for how many kinematic variables do you need to calculate asymmetry?    
nk_asym = NaN(size(nk,1),numvar*2);
nk_asym(:,1:10) = nk(:,1:10);
nk_asym(:,11:12) = nk(:,18:19); % this reproduces the norm_kinematics data matrix without the rotational data, i.e. only the kinematics for the left and right wings
asym = calcAsymmetry(nk_asym,round(w/2));

%%
figure
ix = round(w/2);
shift = 13;
numplot = 11;
x=[0,2.6];
subplot(numplot,1,1)
plot(norm_domain(:,1),nk(:,12),'color',[.75 .75 .75],'linewidth',2), hold on
plot(norm_domain(:,1),nk(:,11),'color',[.1 .1 .1],'linewidth',2)
plot(norm_domain(:,1),nk(:,17)*3,'k')
title({trl;'Body Orientation'}), legend('Pitch','Roll')
xlim(x)
box off, grid on

subplot(numplot,1,2)
plot(norm_domain(:,1),nk(:,3),'b'),hold on
plot(norm_domain(:,1),nk(:,4),'r')
title('Humeral Elevation')
box off, grid on
xlim(x)

subplot(numplot,1,3)
plot(norm_domain(shift:(length(asym)+(shift-1)),1),asym(:,2))
title('Humeral Elevation Asymmetry')
xlim(x)
box off, grid on

subplot(numplot,1,4)
plot(norm_domain(:,1),nk(:,1),'b'),hold on
plot(norm_domain(:,1),nk(:,2),'r')
title('Extension')
xlim(x)
box off, grid on

subplot(numplot,1,5)
plot(norm_domain(shift:(length(asym)+(shift-1)),1),asym(:,1))
title('Extension Asymmetry')
xlim(x)
box off, grid on

subplot(numplot,1,6)
plot(norm_domain(:,1),nk(:,7),'b'),hold on
plot(norm_domain(:,1),nk(:,8),'r')
title('Elbow Extension')
xlim(x)
box off, grid on

subplot(numplot,1,7)
plot(norm_domain(shift:(length(asym)+(shift-1)),1),asym(:,4))
title('Elbow Extension Asymmetry')
xlim(x)
box off, grid on

subplot(numplot,1,8)
plot(norm_domain(:,1),nk(:,9),'b'),hold on
plot(norm_domain(:,1),nk(:,10),'r')
title('Handwing Abduction')
xlim(x)
box off, grid on

subplot(numplot,1,9)
plot(norm_domain(shift:(length(asym)+(shift-1)),1),asym(:,5))
title('Handwing Abduction Asymmetry')
xlim(x)
box off, grid on

subplot(numplot,1,10)
plot(norm_domain(:,1),nk(:,18),'b'),hold on
plot(norm_domain(:,1),nk(:,19),'r')
title('Angle of Attack')
xlim(x)
box off, grid on

subplot(numplot,1,11)
plot(norm_domain(shift:(length(asym)+(shift-1)),1),asym(:,6))
title('Angle of Attack Asymmetry')
xlim(x)
box off, grid on

%% Edit asym to shift data according to shift (accounts for the fact that the
%asymmetry analysis uses a window)
asym_old = asym; clear asym
asym = NaN(length(nk), size(asym_old,2));
asym(shift+1:length(asym_old)+shift,:)  = asym_old;

%% Plot Perturbed Kinematic Variables for pulling data for statistical analysis

% Produce plot?
pl = 'y';

% Which Variable?
v = nk(:,1:2);

% Create wingbeat-normalized time-vector
% nd = [.25:.01:3.5]';

% Plot the Variable for each trial
if pl == 'y'
  lw = 3;  
  figure
  subplot(2,1,1)
  plot(v, 'linewidth',lw)
  legend('Left/Pert','Right/Unpert')
  subplot(2,1,2)
  plot(nk(:,3:4), 'linewidth',lw)
  legend('Left/Pert','Right/Unpert')
end

%% Caclulate means for PERTURBED kinematic variables per each half stroke
% meanext(1,:) = Downstroke 1: 1st half ? first recovery stroke
% meanext(2,:) = Downstroke 1: 2nd half
% meanext(3,:) = Upstroke 1: 1st half
% meanext(4,:) = Upstroke 1: 2nd half
% meanext(5,:) = Downstroke 2: 1st half
% meanext(6,:) = Downstroke 2: 2nd half
% meanext(7,:) = Upstroke 2: 1st half
% meanext(8,:) = Upstroke 2: 2nd half

v = nk(:,2); % Right/unperturbed wing

period = 25:1:50;   meanextUNPERT(1,:) = mean(v(period,:));
period = 50:1:75;   meanextUNPERT(2,:) = mean(v(period,:)); 
period = 75:1:100;  meanextUNPERT(3,:) = mean(v(period,:)); 
period = 100:1:125; meanextUNPERT(4,:) = mean(v(period,:));
period = 125:1:150; meanextUNPERT(5,:) = mean(v(period,:));
period = 150:1:175; meanextUNPERT(6,:) = mean(v(period,:)); 
period = 175:1:200; meanextUNPERT(7,:) = mean(v(period,:)); 
period = 200:1:225; meanextUNPERT(8,:) = mean(v(period,:));

v = nk(:,1); % Left/perturbed wing

period = 25:1:50;   meanextPERT(1,:) = mean(v(period,:));
period = 50:1:75;   meanextPERT(2,:) = mean(v(period,:)); 
period = 75:1:100;  meanextPERT(3,:) = mean(v(period,:)); 
period = 100:1:125; meanextPERT(4,:) = mean(v(period,:));
period = 125:1:150; meanextPERT(5,:) = mean(v(period,:));
period = 150:1:175; meanextPERT(6,:) = mean(v(period,:)); 
period = 175:1:200; meanextPERT(7,:) = mean(v(period,:)); 
period = 200:1:225; meanextPERT(8,:) = mean(v(period,:));

%% Save Data
save('RBg017')
